@@ -2,7 +2,7 @@
 (*                                                                        *)
 (*  This file is part of OcamlGraph.                                      *)
 (*                                                                        *)
-(*  Copyright (C) 2009                                                    *)
+(*  Copyright (C) 2009-2010                                               *)
 (*    CEA (Commissariat  l'nergie Atomique)                             *)
 (*                                                                        *)
 (*  you can redistribute it and/or modify it under the terms of the GNU   *)
@@ -17,8 +17,9 @@
 (*  See the file ../LICENSE for more details.                             *)
 (*                                                                        *)
 (*  Authors:                                                              *)
-(*    - Jean-Denis Koeck (jdkoeck@gmail.com)                              *)
 (*    - Julien Signoles  (Julien.Signoles@cea.fr)                         *)
+(*    - Jean-Denis Koeck (jdkoeck@gmail.com)                              *)
+(*    - Benoit Bataille  (benoit.bataille@gmail.com)                      *)
 (*                                                                        *)
 (**************************************************************************)
 
@@ -35,13 +36,8 @@ open DGraphViewItem
 
 (** Graph widget derived from [GnoCanvas.canvas].
     Support zooming and scrolling. *)
-class ['vertex, 'edge, 'cluster] view:
-  ?delay_node:('vertex -> bool) ->
-  ?delay_edge:('edge -> bool) ->
-  ?delay_cluster:('cluster -> bool) ->
-  GnomeCanvas.canvas Gtk.obj ->
-  ('vertex, 'edge, 'cluster) DGraphModel.abstract_model ->
-object
+class type ['vertex, 'edge, 'cluster] view = object
+
   inherit GnoCanvas.canvas
 
   method model : ('vertex, 'edge, 'cluster) DGraphModel.abstract_model
@@ -64,6 +60,10 @@ object
   method iter_succ_e: ('edge view_item -> unit) -> 'vertex view_item -> unit
   method iter_pred_e: ('edge view_item -> unit) -> 'vertex view_item -> unit
 
+  (* Benoit Bataille's method: is it really useful? *)
+  method iter_associated_vertex:
+    ('vertex view_item -> unit) -> 'vertex view_item -> unit
+
   (** {2 Membership functions} *)
 
   method mem_edge: 'vertex view_item -> 'vertex view_item -> bool
@@ -87,7 +87,7 @@ object
 
   method connect_highlighting_event: unit -> unit
 
-  method highlight: ?color:string * string -> 'vertex view_item -> unit
+  method highlight: ?color: int32 * int32 -> 'vertex view_item -> unit
     (** Change the color of the given vertex item.
 	May be cancelled by [dehighlight].
 	If [color] is [primary,secondary], then
@@ -99,35 +99,28 @@ object
 
 end
 
-val view:
-  ?aa:bool (** Anti-aliasing *) ->
-  ?delay_node:('vertex -> bool) ->
-  ?delay_edge:('edge -> bool) ->
-  ?delay_cluster:('cluster -> bool) ->
-  ?border_width:int ->
-  ?width:int ->
-  ?height:int ->
-  ?packing:(GObj.widget -> unit) ->
-  ?show:bool ->
-  ('vertex, 'edge, 'cluster) DGraphModel.abstract_model ->
-  ('vertex, 'edge, 'cluster) view
-  (** View as a Gnome Canvas.
-      Support zooming and scrolling. *)
+module type S = sig
 
-(* Same widget augmented with highlighting, focus
-    and the ability to drag the canvas (click'n hold)
-*)
-(* class ['vertex, 'edge, 'cluster] drag_view : *)
-(*   GnomeCanvas.canvas Gtk.obj -> *)
-(*   ('vertex, 'edge, 'cluster) DGraphModel.abstract_model -> *)
-(*   ['vertex, 'edge, 'cluster] view *)
+  type vertex
+  type edge
+  type cluster
 
-(* val drag_view : *)
-(*   ?aa:bool -> (\** Anti aliasing *\) *)
-(*   ('vertex, 'edge, 'cluster) DGraphModel.abstract_model -> *)
-(*   ?border_width:int -> *)
-(*   ?width:int -> *)
-(*   ?height:int -> *)
-(*   ?packing:(GObj.widget -> unit) -> *)
-(*   ?show:bool -> unit *)
-(*   -> ('vertex, 'edge, 'cluster) highlight_focus_view *)
+  val view:
+    ?aa:bool (** Anti-aliasing *) ->
+    ?delay_node:(vertex -> bool) ->
+    ?delay_edge:(edge -> bool) ->
+    ?delay_cluster:(cluster -> bool) ->
+    ?border_width:int ->
+    ?width:int ->
+    ?height:int ->
+    ?packing:(GObj.widget -> unit) ->
+    ?show:bool ->
+    (vertex, edge, cluster) DGraphModel.abstract_model ->
+    (vertex, edge, cluster) view
+(** View as a Gnome Canvas.
+    Support zooming and scrolling. *)
+
+end
+
+module Make(V: Sig.HASHABLE)(E: Sig.HASHABLE)(C: Sig.HASHABLE) :
+  S with type vertex = V.t and type edge = E.t and type cluster = C.t
