@@ -2,7 +2,7 @@
 (*                                                                        *)
 (*  This file is part of OcamlGraph.                                      *)
 (*                                                                        *)
-(*  Copyright (C) 2009                                                    *)
+(*  Copyright (C) 2009-2010                                               *)
 (*    CEA (Commissariat  l'nergie Atomique)                             *)
 (*                                                                        *)
 (*  you can redistribute it and/or modify it under the terms of the GNU   *)
@@ -17,8 +17,9 @@
 (*  See the file ../LICENSE for more details.                             *)
 (*                                                                        *)
 (*  Authors:                                                              *)
-(*    - Jean-Denis Koeck (jdkoeck@gmail.com)                              *)
 (*    - Julien Signoles  (Julien.Signoles@cea.fr)                         *)
+(*    - Jean-Denis Koeck (jdkoeck@gmail.com)                              *)
+(*    - Benoit Bataille  (benoit.bataille@gmail.com)                      *)
 (*                                                                        *)
 (**************************************************************************)
 
@@ -27,15 +28,46 @@ open XDot
 open XDotDraw
 open Printf
 
+let red_color =   0xFF0000FFl
+let blue_color =  0x0000FFFFl
+let green_color = 0x00FF00FFl
+let black_color = 0x000000FFl
+let white_color = 0xFFFFFFFFl
+
+
+exception Cannot_convert_color of string
+let rgba_to_int32 col =
+  (* Graphivz sometimes insert named colors by itself, we need to catch
+     those. The list might be incomplete though; if it is, we might need to
+     translate http://www.w3.org/TR/SVG/types.html#ColorKeywords *)
+  match col with
+    | "black" -> black_color
+    | "white" -> white_color
+    | "red" -> red_color
+    | "green" -> green_color
+    | "blue" -> blue_color
+    | _ ->
+        try
+          match String.length col with
+            | 7 -> Graph.Graphviz.color_to_color_with_transparency
+                (int_of_string ("0x" ^ String.sub col 1 6))
+            | 9 -> Int32.of_string ("0x" ^ String.sub col 1 8)
+            | _ -> failwith ""
+        with Failure _ ->
+          (* Can fail either in the failed match above, or in int*of_string *)
+          raise (Cannot_convert_color (col^" but int32 expected"))
+
+let convert_fill_color col = `FILL_COLOR_RGBA (rgba_to_int32 col)
+let convert_outline_color col = `OUTLINE_COLOR_RGBA (rgba_to_int32 col)
+
 let ($) f x = f x
 
-let red_color = "#FF0000"
-let green_color = "#00FF00"
 
 (* Derived text class. *)
 class graph_text txt_obj ~size_points ~(props:GnomeCanvas.text_p list) =
   let props = `SIZE_POINTS size_points :: props in
 object (self)
+
   inherit GnoCanvas.text txt_obj as text
 
   val mutable props = props
@@ -44,31 +76,29 @@ object (self)
     props <- p;
     text#set p
 
-  method init_size = size_points
-
   method highlight ?(color=red_color,green_color) () =
     let primary, secondary = color in
     let color = ref primary in
     let rec hi_props = function
       | [] -> []
       | `SIZE_POINTS p :: l ->
-	  let s = if p >= 12. then p else max 6. (p *. 1.5) in
-	  `SIZE_POINTS s :: hi_props l
+	let p = if p >= 12. then p else max 6. (p *. 1.5) in
+	`SIZE_POINTS p :: hi_props l
       | `WEIGHT d :: l ->
-	  `WEIGHT (max 600 (int_of_float (float d *. 1.5))) :: hi_props l
-      | `FILL_COLOR c :: l ->
-	  if c = primary then color := secondary;
-	  hi_props l
+	`WEIGHT (max 600 (int_of_float (float d *. 1.5))) :: hi_props l
+      | `FILL_COLOR_RGBA c :: l ->
+	if c = primary then color := secondary;
+	hi_props l
       | p :: l -> p :: hi_props l
     in
     (* as inserted in head, `WEIGHT 600 will not apply if there is already a
        specified weight *)
-    text#set (`FILL_COLOR !color :: `WEIGHT 600 :: hi_props props)
+    text#set (`FILL_COLOR_RGBA !color :: `WEIGHT 600 :: hi_props props)
 
   method dehighlight () =
     (* as inserted in head, properties will not apply if they are already
        specified *)
-    text#set (`WEIGHT 400 :: `FILL_COLOR "black" :: props)
+    text#set (`WEIGHT 400 :: `FILL_COLOR_RGBA black_color :: props)
 
   method resize f =
     let rec change = function
@@ -79,7 +109,8 @@ object (self)
     in
     self#set (change props)
 
-  initializer text#set props
+  initializer
+    text#set props
 
 end
 
@@ -101,8 +132,8 @@ let graph_text ?x ?y ?text ?font ?anchor ~size_points ?(props=[]) p =
 (* Shape properties.
    Used when initializing items *)
 
-let pen_color draw_st = `OUTLINE_COLOR draw_st.XDotDraw.pen_color
-let fill_color draw_st = `FILL_COLOR draw_st.XDotDraw.fill_color
+let pen_color draw_st = convert_outline_color draw_st.XDotDraw.pen_color
+let fill_color draw_st = convert_fill_color draw_st.XDotDraw.fill_color
 
 (* Flattens an array of pair of coordinates into an array of coordinates *)
 let flatten_points pts =
@@ -130,8 +161,8 @@ let shape_item = function
 
 (* Shape properties (used by canvas items ellipse, polygon and bpath) *)
 type shape_p =
-    [ `FILL_COLOR of string
-    | `OUTLINE_COLOR of string
+    [ `FILL_COLOR_RGBA of int32
+    | `OUTLINE_COLOR_RGBA of int32
     | `WIDTH_UNITS of float
     | `DASH of float * float array ]
 
@@ -146,17 +177,19 @@ let to_p = function
 let complete_props props =
   let rec what_props (fill, width, outline, dash as acc) = function
     | [] -> acc
-    | `FILL_COLOR _ :: l -> what_props (true, width, outline, dash) l
+    | `FILL_COLOR_RGBA _ :: l -> what_props (true, width, outline, dash) l
     | `WIDTH_UNITS _ :: l -> what_props (fill, true, outline, dash) l
-    | `OUTLINE_COLOR _ :: l -> what_props (fill, width, true, dash) l
+    | `OUTLINE_COLOR_RGBA _ :: l -> what_props (fill, width, true, dash) l
     | `DASH _ :: l -> what_props (fill, width, outline, true) l
   in
   let fill, width, outline, dash =
     what_props (false, false, false, false) props
   in
-  let props = if fill then props else `FILL_COLOR "black" :: props in
+  let props = if fill then props else `FILL_COLOR_RGBA black_color :: props in
   let props = if width then props else `WIDTH_UNITS 1. :: props in
-  let props = if outline then props else `OUTLINE_COLOR "black" :: props in
+  let props =
+    if outline then props else `OUTLINE_COLOR_RGBA black_color :: props
+  in
   if dash then props else `DASH (0., [| |]) :: props
 
 (* Shape class (either a rect, an ellipse, a polygon or a path).
@@ -183,10 +216,10 @@ class shape ~fill shape init_props = object (self)
     let rec hi_props = function
       | [] -> []
       | `WIDTH_UNITS u :: l -> `WIDTH_UNITS (max 3. (u *. 3.)) :: hi_props l
-      | `FILL_COLOR c :: l when fill ->
+      | `FILL_COLOR_RGBA c :: l when fill ->
 	  if c = primary then color := secondary;
 	  hi_props l
-      | `OUTLINE_COLOR c :: l ->
+      | `OUTLINE_COLOR_RGBA c :: l ->
 	  if c = primary then color := secondary;
 	  hi_props l
       | p :: l -> p :: hi_props l
@@ -194,14 +227,14 @@ class shape ~fill shape init_props = object (self)
     (* as inserted in head, `WIDTH_UNITS 3. will not apply if there is already
        a specified weight *)
     let props = hi_props props in
-    let props = if fill then `FILL_COLOR !color :: props else props in
-    self#set_props (`OUTLINE_COLOR !color :: `WIDTH_UNITS 3. :: props)
+    let props = if fill then `FILL_COLOR_RGBA !color :: props else props in
+    self#set_props (`OUTLINE_COLOR_RGBA !color :: `WIDTH_UNITS 3. :: props)
 
   method dehighlight () =
     (* as inserted in head, `WIDTH_UNITS 1. will not apply if there is already
        a  specified weight *)
-    let props = if fill then `FILL_COLOR "black" :: props else props in
-    self#set_props (`WIDTH_UNITS 1. :: `OUTLINE_COLOR "black" :: props)
+    let props = if fill then `FILL_COLOR_RGBA black_color :: props else props in
+    self#set_props (`WIDTH_UNITS 1. :: `OUTLINE_COLOR_RGBA black_color :: props)
 
   initializer self#set (complete_props init_props)
 
@@ -255,14 +288,17 @@ let bspline ~fill draw_st group pts =
   let bpath = GnoCanvas.bpath group ~bpath:path ~props in
   new shape ~fill (SBSpline bpath) props
 
-let text draw_st group pos align label =
+let text draw_st group pos align anchor label =
   let size_points, font = draw_st.XDotDraw.font in
   let x, y = XDot.conv_coord pos in
   let y = y +. size_points /. 2. in
-  let props = [ `FILL_COLOR draw_st.XDotDraw.pen_color ] in
+  let props = [ convert_fill_color draw_st.XDotDraw.pen_color ] in
+  let anchor =
+    if anchor = -. 1. then `WEST else if anchor = 1.0 then `EAST else `CENTER
+  in
   graph_text
     group
-    ~x ~y ~text:label ~props ~anchor:`SOUTH
+    ~x ~y ~text:label ~props ~anchor
     ~font ~size_points:(size_points -. 1.)
 
 class type common_view = object
@@ -274,7 +310,7 @@ end
 (* ITEMS *)
 
 class type textshape = object
-  method highlight: ?color:string * string -> unit -> unit
+  method highlight: ?color:int32 * int32 -> unit -> unit
   method dehighlight: unit -> unit
   method hide: unit -> unit
   method show: unit -> unit
@@ -300,7 +336,7 @@ object (self)
   inherit GnoCanvas.group view#root#as_group
 
   val mutable hilighted = false
-  val mutable text_opt = None
+  val mutable texts = []
   val mutable shapes = []
   val mutable computed = not delay
   val mutable cached_events = []
@@ -312,36 +348,32 @@ object (self)
       if computed then f x
       else cached_events <- (fun () -> f x) :: cached_events
 
-  method zoom_text zf =
+  method zoom_text (zf:float) =
     self#cache
-      (fun zf -> match text_opt with
-       | None -> ()
-       | Some t ->
-	   let new_size = t#init_size *. zf in
-	   t#resize new_size)
+      (fun zf -> List.iter (fun t -> t#resize (10. *. zf)) texts)
       zf
 
   method private iter f =
-    (match text_opt with None -> () | Some t -> f (t :> textshape));
+    List.iter (fun t -> f (t :> textshape)) texts;
     List.iter f (shapes :> textshape list)
 
   method highlight ?color () =
     self#cache
       (fun () ->
-	 if not hilighted then begin
-	   hilighted <- true;
-	   self#iter (fun s -> s#highlight ?color ());
-	   match text_opt with None -> () | Some t -> t#raise_to_top ();
-	 end)
+	if not hilighted then begin
+	  hilighted <- true;
+	  self#iter (fun s -> s#highlight ?color ());
+	  List.iter (fun t -> t#raise_to_top ()) texts;
+	end)
       ()
 
   method dehighlight () =
     self#cache
       (fun () ->
-	 if hilighted then begin
-	   hilighted <- false;
-	   self#iter (fun s -> s#dehighlight ());
-	 end)
+	if hilighted then begin
+	  hilighted <- false;
+	  self#iter (fun s -> s#dehighlight ());
+	end)
       ()
 
   method hide () = self#cache (fun () -> self#iter (fun s -> s#hide ())) ()
@@ -355,13 +387,13 @@ object (self)
   method center () =
     self#cache
       (fun () ->
-	 let x, y = pos in
-	 let w = view#hadjustment#page_size /. view#zoom_factor in
-	 let h = view#vadjustment#page_size /. view#zoom_factor in
-	 let sx = x -. (w /. 2.) in
-	 let sy = y -. (h /. 2.) in
-	 let sx, sy = view#w2c ~wx:sx ~wy:sy in
-	 ignore $ view#scroll_to ~x:sx ~y:sy)
+	let x, y = pos in
+	let w = view#hadjustment#page_size /. view#zoom_factor in
+	let h = view#vadjustment#page_size /. view#zoom_factor in
+	let sx = x -. (w /. 2.) in
+	let sy = y -. (h /. 2.) in
+	let sx, sy = view#w2c ~wx:sx ~wy:sy in
+	ignore $ view#scroll_to ~x:sx ~y:sy)
       ()
 
   method lower_to_bottom () =
@@ -371,20 +403,20 @@ object (self)
      Updates the shapes and texts lists *)
   method private read_operations () =
     let read_op draw_st = function
-	(* Create shapes *)
+      (* Create shapes *)
       | XDotDraw.Filled_ellipse (pos, w, h)
       | XDotDraw.Unfilled_ellipse (pos, w, h) ->
-	  shapes <- ellipse ~fill draw_st self pos w h :: shapes
+	shapes <- ellipse ~fill draw_st self pos w h :: shapes
       | XDotDraw.Filled_polygon pts | XDotDraw.Unfilled_polygon pts ->
-	  shapes <- polygon ~fill draw_st self pts :: shapes
+	shapes <- polygon ~fill draw_st self pts :: shapes
       | XDotDraw.Bspline pts | XDotDraw.Filled_bspline pts ->
-	  shapes <- bspline ~fill draw_st self pts :: shapes
-      | XDotDraw.Text (pos, align, _, label) ->
-	  text_opt <- Some (text draw_st self pos align label)
+	shapes <- bspline ~fill draw_st self pts :: shapes
+      | XDotDraw.Text (pos, align, anchor, label) ->
+	texts <- text draw_st self pos align anchor label :: texts
       | _ -> ()
     in
     List.iter (draw_with read_op) ops_list;
-    (match text_opt with None -> () | Some t -> t#raise_to_top ());
+    List.iter (fun t -> t#raise_to_top ()) texts;
     List.fold_right (* preserve order *) (fun f () -> f ()) cached_events ()
 
   method compute () =
